package tictactoe;

import java.io.*;

/**
 * Instantiates a plain ASCII version of TicTacToe or Gomoku.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.0 1999-03-26
 */
public class GameDriver {
	/**
	 * Queries the user whether to instantiate TicTacToe or Gomoku.
	 */
	public static void main(String args[]) {
		StreamPlayer X = new StreamPlayer('X');
		StreamPlayer O = new StreamPlayer('O');
		
		System.out.print("Hi!  Would you like to play TicTacToe (t) or Gomoku (g)?: ");
		
		switch(readChar()) {
		case 't':
			playGame(new TicTacToe(X, O));
			break;
		case 'g':
			playGame(new Gomoku(X, O));
			break;
		default:
			System.out.println("I guess you don't want to play!");
		}
		System.out.println("bye!");
	}

	/**
	 * Plays an instantiated BoardGame in verbose mode
	 * (i.e., printing every board state).
	 */
	public static void playGame(BoardGame game) {
		playGame(game, true);
	}
	
	/**
	 * Helper method to get a character from the standard input.
	 */
	public static char readChar() {
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
			String line = in.readLine();
			if ((line == null) || (line.length() == 0))
				return '\n';
			else
				return line.charAt(0);
		} catch (IOException err) {
			return '\n';
		}
	}
	
	/**
	 * Plays a game until it is over by repeatedly asking the current Player to move.
	 * Can also be used in silent mode to run a test suite.  
	 * @param game the game to play
	 * @param verbose boolean flag to print game state if true
	 */
	public static void playGame(BoardGame game, boolean verbose) {
		try {
			do {
				if (verbose) {
					System.out.println();
					System.out.println(game);
					System.out.print("Player "
						+ game.currentPlayer().mark() + " moves: ");
				}
				((StreamPlayer) game.currentPlayer()).move();
			} while(game.notOver());
			if (verbose) {
				System.out.print(game);
				System.out.println("game over -- " + game.winner() + " wins!");
			}
		} catch (IOException err) {
			if (verbose)
				System.out.println("game terminated!");
		} catch (Exception err) {
			err.printStackTrace();
			System.exit(1);
		}
	}
}
