package tictactoe;

import java.io.*;

/**
 * Bundles static methods to instantiate TicTacToe
 * or Gomoku on the server side.  Since there is only
 * a single server, we have made all methods and instance
 * variables static.  (Perhaps it would be better to
 * make this a Singleton class?)
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.1 1999-03-31
 */

public class GameFactory {
	
	static private RemoteGame _game;

	/**
	 * Returns TicTacToe instance for a remote client.
	 * If a game already exists, the client joins the
	 * existing game.  Else a new game is made.
	 */
	public static synchronized RemoteGame joinGame() {
		RemoteGame game = _game;
		if (game == null) {
			// game =  new TicTacToe(newPlayer('X'), newPlayer('O'));
			game =  new Gomoku(newPlayer('X'), newPlayer('O'), 10, 10);
			_game = game;
		} else {
			_game = null;
		}
		return game;
	}

	private static Player newPlayer(char mark) {
		return new PassivePlayer(mark);
	}
}