package tictactoe;

import java.io.*;

/**
 * Helper class to find winning scores for BoardGame.
 * A Runner runs across a BoardGame, starting at the
 * position last played, and searches for a winning score.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.6 1999-02-04
 */
public class Runner {

	BoardGame _game;
	// Home col and row:
	int _homeCol;
	int _homeRow;
	// Current col & row:
	int _col=0;
	int _row=0;

	/**
	 * @param game the associated BoardGame
	 * @param col the column of the last position played
	 * @param row the row of the last position played
	 */
	public Runner(BoardGame game, int col, int row)
	{
		_game = game;
		_homeCol = col;
		_homeRow = row;
	}

	/**
	 * A Runner runs in some direction (dcol,drow)
	 * as far as it can, as long as the pieces on the
	 * Board are the same as the home square.
	 * Then it runs in the opposite direction, and
	 * returns the total score.
	 */
	public int run(int dcol, int drow)
		throws AssertionException
	{
		int score = 1;
		this.goHome();
		score += this.forwardRun(dcol, drow);
		this.goHome();
		dcol = -dcol;
		drow = -drow;
		score += this.forwardRun(dcol, drow);
		debug("Runner(" + _homeCol + "," + _homeRow + ")" +
			".run(" + -dcol + "," + -drow + ") = " + score + "\n");
		return score;
	}
	
	/**
	 * Return to the "home" position (the last one played).
	 */
	private void goHome() {
		_col= _homeCol;
		_row = _homeRow;
	}
	
	/**
	 * Recursively move forward as long as we are
	 * in a run.  Return the length of the run.
	 * @param dcol increment in column to move
	 * @param drow increment in row to move
	 */
	private int forwardRun(int dcol, int drow)
		throws AssertionException
	{
		this.move(dcol, drow);
		if (this.samePlayer())
			return 1 + this.forwardRun(dcol, drow);
		else
			return 0;
	}
	
	/**
	 * Move in the direction given by the increments.
	 */
	private void move(int dcol, int drow) {
		_col = _col + dcol;
		_row = _row + drow;
		debug("Runner.move(" + _col + "," + _row + ")\n");
	}

	/**
	 * The player at the current location is the
	 * same as that of our home square.
	 */
	private boolean samePlayer()
		throws AssertionException
	{
		if (_game.inRange(_col,_row)) {
			Player home = _game.get(_homeCol, _homeRow);
			Player here = _game.get(_col, _row);
			return home == here;
		} else {
			// Oops -- we walked off the edge of the board!
			return false;
		}
	}
	
	// Introduced this to fugure out why interactive
	// 10*5 Gomoku disn't find a winner after 5 in a row.
	// (Problem was GameApplet.makeGrid nested loop was backwards.)
	private void debug(String msg) {
		// System.err.print(msg);
	}
}
